@@ -63,17 +63,16 @@ features could also be easily plugged into a custom top controller.
 before 'COMPONENT' => sub {
   my $class = shift;
   my $app_class = ref $_[0] || $_[0];
-  
-  my $ns = $app_class->module_root_namespace;
-  $class->config( namespace => $ns );
-};
 
+  my $module_root_namespace = $app_class->module_root_namespace;
+  $class->config( namespace => $module_root_namespace || '' );
+};
 
 sub process { (shift)->approot(@_) }
 
-sub approot :Path {
+sub approot :Chained :PathPrefix :Args {
   my ($self, $c, @args) = @_;
-  
+
   # Handle url string mode:
   if(scalar(@args) == 1 && $args[0] =~ /^\//) {
     my $url = $args[0];
@@ -89,7 +88,6 @@ sub approot :Path {
   $self->dispatch($c, @args);
 }
 
-
 sub end : ActionClass('RenderView') {}
 
 no Moose;
