@@ -105,17 +105,6 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
 		};
 		this.cmp.on('render',cmp.bubbleTieStoreParents,cmp);
 		
-		
-		this.cmp.origInitEvents = this.cmp.initEvents;
-		this.cmp.initEvents = function() {
-			cmp.origInitEvents.call(cmp);
-			if(cmp.loadMask){
-				cmp.loadMask = new Ext.LoadMask(cmp.bwrap,
-						  Ext.apply({store:cmp.store}, cmp.loadMask));
-			}
-		}
-		
-		
 		// -- instead of standard store 'autoLoad' param, 'store_autoLoad' happens
 		// on 'render' in order to be delayed, in the case of the component being
 		// within a collapsed panel, etc. This is partnered with the setting set
@@ -177,6 +166,10 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
 		this.exclude_btn_map = {};
 		Ext.each(this.store_exclude_buttons,function(item) { this.exclude_btn_map[item] = true; },this);
 		if(!this.use_edit_form) { this.exclude_btn_map.edit = true; }
+    if(cmp.persist_all_immediately) {
+      this.exclude_btn_map.save = true;
+      this.exclude_btn_map.undo = true;
+    }
 		
 		this.initAdditionalStoreMethods.call(this,this.cmp.store,this);
 		
@@ -208,10 +201,12 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
 		if(this.cmp.setup_bbar_store_buttons) {
 			this.cmp.on('render',this.insertStoreButtonsBbar,this);
 		}
-		
-		if(this.cmp.setup_tbar_store_buttons) {
+		else if(this.cmp.setup_tbar_store_buttons) {
 			this.cmp.on('render',this.insertStoreButtonsTbar,this);
 		}
+    else {
+      this.cmp.on('render',this.initializeStoreButtons,this);
+    }
 		
 		// Only applies to editor grids; no effect/impact on other components
 		// without the beforeedit/afteredit events
@@ -310,17 +305,61 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
       // are slow/bogged down, but I am not 100% sure...
 			},cmp.store,{ delay: 10 });
 		}
+    
+    cmp.addEvents('firstload');
+    
+    cmp.on('render',function() {
+      var El = Ext.isFunction(cmp.getLoadMaskEl) // <-- Allow the cmp to define (AppDV)
+        ? cmp.getLoadMaskEl() : cmp.getEl() || Ext.getBody();
+      
+      // -- Display a page-wide mask during save
+      if(this.store_write_mask) {
+        var myMask = new Ext.LoadMask(El, {msg:"Saving Changes..."});
+        var show_mask = function() { myMask.show(); }
+        var hide_mask = function() { myMask.hide(); }
+        
+        cmp.store.on('beforewrite',show_mask,this);
+        cmp.store.on('write',hide_mask,this);
+        cmp.store.on('exception',hide_mask,this);
+      }
+      
+      if(cmp.loadMask) {
+        // We want to delete loadMask to prevent the built-in logic in grid
+        // from initializing since we're handing it in a common manner
+        delete cmp.loadMask;
+        cmp.readMask = new Ext.LoadMask(El, {store:cmp.store});
+      
+        // ---- Workaround - manual single-use loadMask for the very first load
+        // Need to investigate more why this is needed, and why the 'loadMask' grid
+        // setting doesn't work on the first store load. I think it is related to
+        // load order and possibly autoPanel. 
+
+        // Check to make sure store_autoLoad has not been set to a false value in
+        // either the store or the grid config (which is now allowed to override the
+        // store setting, see datastore-plus code)
+        var store_autoLoad_disabled = (
+          (typeof cmp.store_autoLoad != 'undefined' && !cmp.store_autoLoad) || 
+          !cmp.store.store_autoLoad
+        ) ? true : false;
+      
+        if(!cmp.collapsed && !store_autoLoad_disabled) {
+          var lMask = new Ext.LoadMask(El,{ msg: "Loading Data Set" });
+          lMask.show();
+          var hide_fn;
+          hide_fn = function(){ 
+            cmp.fireEvent('firstload');
+            lMask.hide(); 
+            cmp.store.un('load',hide_fn);
+            cmp.store.un('exception',hide_fn); 
+          };
+          cmp.store.on('load',hide_fn,this);
+          cmp.store.on('exception',hide_fn,this);
+        }
+        // ----
+      }
+      
+    },this);
 		
-		// -- Display a page-wide mask during save
-		if(this.store_write_mask) {
-			var myMask = new Ext.LoadMask(Ext.getBody(), {msg:"Saving Changes..."});
-			var show_mask = function() { myMask.show(); }
-			var hide_mask = function() { myMask.hide(); }
-			
-			cmp.store.on('beforewrite',show_mask,this);
-			cmp.store.on('write',hide_mask,this);
-			cmp.store.on('exception',hide_mask,this);
-		}
 		// --
 		
 		
@@ -777,6 +816,9 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
 				var sm = plugin.cmp.getSelectionModel();
 				count = sm.getCount();
 			}
+      else if(plugin.cmp.getSelectionCount) { // AppDV
+        count = plugin.cmp.getSelectionCount();
+      }
 			else {
 				count = store.getCount();
 			}
@@ -1137,10 +1179,20 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
 	},
 	
 	getStoreButton: function(name,showtext) {
-		
-		if(this.exclude_btn_map[name]) { return; }
-		
-		if(!this.cmp.loadedStoreButtons[name]) {
+    var El = this.cmp.el;
+    if(typeof this.cmp.loadedStoreButtons[name] == "undefined") {
+    
+      if(this.exclude_btn_map[name]) {
+        if(El) {
+          if(El.hasClass('ra-dsbtn-allow-'+name)) {
+            El.removeClass('ra-dsbtn-allow-'+name);
+          }
+          El.addClass('ra-dsbtn-deny-'+name);
+        }
+        this.cmp.loadedStoreButtons[name] = null;
+        return null; 
+      }
+    
 			var constructor = this.getStoreButtonConstructors.call(this)[name];
 			if(! constructor) { return; }
 			
@@ -1151,6 +1203,28 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
 			
 			var btn = constructor(cnf,this.cmp,showtext);
 			if(!btn) { return; }
+      
+      // ----------
+      
+      if(El) {
+        btn.on('enable',function(b){
+          if(El.hasClass('ra-dsbtn-deny-'+name)) {
+            El.removeClass('ra-dsbtn-deny-'+name);
+          }
+          El.addClass('ra-dsbtn-allow-'+name);
+        },this);
+        
+        btn.on('disable',function(b){
+          if(El.hasClass('ra-dsbtn-allow-'+name)) {
+            El.removeClass('ra-dsbtn-allow-'+name);
+          }
+          El.addClass('ra-dsbtn-deny-'+name);
+        },this);
+      }
+      // Ensure the event is fired to set the correct initial state:
+      btn.disabled ? btn.fireEvent('disable',btn) : btn.fireEvent('enable',btn);
+      // ----------
+      
 			
 			this.cmp.loadedStoreButtons[name] = btn;
 			
@@ -1400,6 +1474,16 @@ Ext.ux.RapidApp.Plugin.CmpDataStorePlus = Ext.extend(Ext.util.Observable,{
 			}
 		};
 	},
+  
+  // Alternative, pure initialization if we're not calling 
+  // insertStoreButtonsBbar/insertStoreButtonsTbar
+  initializeStoreButtons: function() {
+    var showtext = this.show_store_button_text ? true : false;
+    Ext.each(this.store_buttons,function(btn_name) {
+      this.getStoreButton(btn_name,showtext);
+    },this);
+  },
+  
 	
 	insertStoreButtonsBbar: function() {
 		var index = 0;
